from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level, cast_uint
import WebKit
import objc

UINT_MAX = cast_uint(-1)


class TestWebUIDelegateHelper(WebKit.NSObject):
    def webView_willPerformDragDestinationAction_forDraggingInfo_(self, wv, a, i):
        pass

    def webViewAreToolbarsVisible_(self, a):
        return 1

    def webView_setToolbarsVisible_(self, a, b):
        pass

    def webViewIsStatusBarVisible_(self, a):
        return 1

    def webView_setStatusBarVisible_(self, a, b):
        pass

    def webViewIsResizable_(self, a):
        return 1

    def webView_setResizable_(self, a, b):
        pass

    def webView_runJavaScriptConfirmPanelWithMessage_initiatedByFrame_(self, a, b, c):
        return 1

    def webView_runBeforeUnloadConfirmPanelWithMessage_initiatedByFrame_(self, a, b, c):
        return 1

    def webView_mouseDidMoveOverElement_modifierFlags_(self, a, b, c):
        pass

    def webView_validateUserInterfaceItem_defaultValidation_(self, a, b, c):
        return 1

    def webView_shouldPerformAction_fromSender_(self, a, b, c):
        return 1

    def webView_dragDestinationActionMaskForDraggingInfo_(self, a, b):
        return 1

    def webView_dragSourceActionMaskForPoint_(self, a, b):
        return 1

    def webView_willPerformDragSourceAction_fromPoint_withPasteboard_(self, a, b, c, d):
        pass

    def webView_printFrameView_(self, a, b):
        pass

    def webViewHeaderHeight_(self, a):
        return 1

    def webViewFooterHeight_(self, a):
        return 1

    def webView_drawHeaderInRect_(self, a, b):
        pass

    def webView_drawFooterInRect_(self, a, b):
        pass

    def webView_runJavaScriptConfirmPanelWithMessage_(self, a, b):
        return 1

    def webView_setContentRect_(self, a, b):
        pass

    def webViewContentRect_(self, a):
        return 1

    def webView_runOpenPanelForFileButtonWithResultListener_allowMultipleFiles_(
        self, a, b, c
    ):
        pass


class TestWebUIDelegate(TestCase):
    def testConstants(self):
        self.assertEqual(WebKit.WebMenuItemTagOpenLinkInNewWindow, 1)
        self.assertEqual(WebKit.WebMenuItemTagDownloadLinkToDisk, 2)
        self.assertEqual(WebKit.WebMenuItemTagCopyLinkToClipboard, 3)
        self.assertEqual(WebKit.WebMenuItemTagOpenImageInNewWindow, 4)
        self.assertEqual(WebKit.WebMenuItemTagDownloadImageToDisk, 5)
        self.assertEqual(WebKit.WebMenuItemTagCopyImageToClipboard, 6)
        self.assertEqual(WebKit.WebMenuItemTagOpenFrameInNewWindow, 7)
        self.assertEqual(WebKit.WebMenuItemTagCopy, 8)
        self.assertEqual(WebKit.WebMenuItemTagGoBack, 9)
        self.assertEqual(WebKit.WebMenuItemTagGoForward, 10)
        self.assertEqual(WebKit.WebMenuItemTagStop, 11)
        self.assertEqual(WebKit.WebMenuItemTagReload, 12)
        self.assertEqual(WebKit.WebMenuItemTagCut, 13)
        self.assertEqual(WebKit.WebMenuItemTagPaste, 14)
        self.assertEqual(WebKit.WebMenuItemTagSpellingGuess, 15)
        self.assertEqual(WebKit.WebMenuItemTagNoGuessesFound, 16)
        self.assertEqual(WebKit.WebMenuItemTagIgnoreSpelling, 17)
        self.assertEqual(WebKit.WebMenuItemTagLearnSpelling, 18)
        self.assertEqual(WebKit.WebMenuItemTagOther, 19)
        self.assertEqual(WebKit.WebMenuItemTagSearchInSpotlight, 20)
        self.assertEqual(WebKit.WebMenuItemTagSearchWeb, 21)
        self.assertEqual(WebKit.WebMenuItemTagLookUpInDictionary, 22)
        self.assertEqual(WebKit.WebMenuItemTagOpenWithDefaultApplication, 23)
        self.assertEqual(WebKit.WebMenuItemPDFActualSize, 24)
        self.assertEqual(WebKit.WebMenuItemPDFZoomIn, 25)
        self.assertEqual(WebKit.WebMenuItemPDFZoomOut, 26)
        self.assertEqual(WebKit.WebMenuItemPDFAutoSize, 27)
        self.assertEqual(WebKit.WebMenuItemPDFSinglePage, 28)
        self.assertEqual(WebKit.WebMenuItemPDFFacingPages, 29)
        self.assertEqual(WebKit.WebMenuItemPDFContinuous, 30)
        self.assertEqual(WebKit.WebMenuItemPDFNextPage, 31)
        self.assertEqual(WebKit.WebMenuItemPDFPreviousPage, 32)

        self.assertEqual(WebKit.WebDragDestinationActionNone, 0)
        self.assertEqual(WebKit.WebDragDestinationActionDHTML, 1)
        self.assertEqual(WebKit.WebDragDestinationActionEdit, 2)
        self.assertEqual(WebKit.WebDragDestinationActionLoad, 4)
        self.assertEqual(WebKit.WebDragDestinationActionAny, UINT_MAX)

        self.assertEqual(WebKit.WebDragSourceActionNone, 0)
        self.assertEqual(WebKit.WebDragSourceActionDHTML, 1)
        self.assertEqual(WebKit.WebDragSourceActionImage, 2)
        self.assertEqual(WebKit.WebDragSourceActionLink, 4)
        self.assertEqual(WebKit.WebDragSourceActionSelection, 8)
        self.assertEqual(WebKit.WebDragSourceActionAny, UINT_MAX)

    def testProtocols(self):
        objc.protocolNamed("WebOpenPanelResultListener")

    @min_sdk_level("10.11")
    def testProtocols10_11(self):
        objc.protocolNamed("WebUIDelegate")

    def testMethods(self):
        self.assertResultIsBOOL(TestWebUIDelegateHelper.webViewAreToolbarsVisible_)
        self.assertArgIsBOOL(TestWebUIDelegateHelper.webView_setToolbarsVisible_, 1)
        self.assertResultIsBOOL(TestWebUIDelegateHelper.webViewIsStatusBarVisible_)
        self.assertArgIsBOOL(TestWebUIDelegateHelper.webView_setStatusBarVisible_, 1)
        self.assertResultIsBOOL(TestWebUIDelegateHelper.webViewIsResizable_)
        self.assertArgIsBOOL(TestWebUIDelegateHelper.webView_setResizable_, 1)
        self.assertResultIsBOOL(
            TestWebUIDelegateHelper.webView_runJavaScriptConfirmPanelWithMessage_initiatedByFrame_  # noqa: B950
        )
        self.assertResultIsBOOL(
            TestWebUIDelegateHelper.webView_runBeforeUnloadConfirmPanelWithMessage_initiatedByFrame_  # noqa: B950
        )
        self.assertArgHasType(
            TestWebUIDelegateHelper.webView_mouseDidMoveOverElement_modifierFlags_,
            2,
            objc._C_NSUInteger,
        )
        self.assertResultIsBOOL(
            TestWebUIDelegateHelper.webView_validateUserInterfaceItem_defaultValidation_
        )
        self.assertArgIsBOOL(
            TestWebUIDelegateHelper.webView_validateUserInterfaceItem_defaultValidation_,
            2,
        )
        self.assertResultIsBOOL(
            TestWebUIDelegateHelper.webView_shouldPerformAction_fromSender_
        )
        self.assertResultHasType(
            TestWebUIDelegateHelper.webView_dragDestinationActionMaskForDraggingInfo_,
            objc._C_NSUInteger,
        )
        self.assertResultHasType(
            TestWebUIDelegateHelper.webView_dragSourceActionMaskForPoint_,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestWebUIDelegateHelper.webView_willPerformDragSourceAction_fromPoint_withPasteboard_,  # noqa: B950
            1,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestWebUIDelegateHelper.webView_willPerformDragSourceAction_fromPoint_withPasteboard_,  # noqa: B950
            2,
            WebKit.NSPoint.__typestr__,
        )
        self.assertResultHasType(
            TestWebUIDelegateHelper.webViewHeaderHeight_, objc._C_FLT
        )
        self.assertResultHasType(
            TestWebUIDelegateHelper.webViewFooterHeight_, objc._C_FLT
        )
        self.assertArgHasType(
            TestWebUIDelegateHelper.webView_drawHeaderInRect_,
            1,
            WebKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestWebUIDelegateHelper.webView_drawFooterInRect_,
            1,
            WebKit.NSRect.__typestr__,
        )
        self.assertResultIsBOOL(
            TestWebUIDelegateHelper.webView_runJavaScriptConfirmPanelWithMessage_
        )
        self.assertArgHasType(
            TestWebUIDelegateHelper.webView_setContentRect_,
            1,
            WebKit.NSRect.__typestr__,
        )
        self.assertResultHasType(
            TestWebUIDelegateHelper.webViewContentRect_, WebKit.NSRect.__typestr__
        )
        self.assertArgHasType(
            TestWebUIDelegateHelper.webView_willPerformDragDestinationAction_forDraggingInfo_,
            1,
            objc._C_NSUInteger,
        )

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgIsBOOL(
            TestWebUIDelegateHelper.webView_runOpenPanelForFileButtonWithResultListener_allowMultipleFiles_,  # noqa: B950
            2,
        )
        self.assertResultIsBOOL(
            TestWebUIDelegateHelper.webView_runJavaScriptConfirmPanelWithMessage_
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestDOMKeyboardEvent(TestCase):
    def testConstants(self):
        self.assertEqual(WebKit.DOM_KEY_LOCATION_STANDARD, 0)
        self.assertEqual(WebKit.DOM_KEY_LOCATION_LEFT, 1)
        self.assertEqual(WebKit.DOM_KEY_LOCATION_RIGHT, 2)
        self.assertEqual(WebKit.DOM_KEY_LOCATION_NUMPAD, 3)

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertResultIsBOOL(WebKit.DOMKeyboardEvent.altGraphKey)

    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMKeyboardEvent.ctrlKey)
        self.assertResultIsBOOL(WebKit.DOMKeyboardEvent.shiftKey)
        self.assertResultIsBOOL(WebKit.DOMKeyboardEvent.altKey)
        self.assertResultIsBOOL(WebKit.DOMKeyboardEvent.metaKey)
        self.assertResultIsBOOL(WebKit.DOMKeyboardEvent.getModifierState_)

        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_keyLocation_ctrlKey_altKey_shiftKey_metaKey_altGraphKey_,  # noqa: B950
            1,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_keyLocation_ctrlKey_altKey_shiftKey_metaKey_altGraphKey_,  # noqa: B950
            2,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_keyLocation_ctrlKey_altKey_shiftKey_metaKey_altGraphKey_,  # noqa: B950
            6,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_keyLocation_ctrlKey_altKey_shiftKey_metaKey_altGraphKey_,  # noqa: B950
            7,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_keyLocation_ctrlKey_altKey_shiftKey_metaKey_altGraphKey_,  # noqa: B950
            8,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_keyLocation_ctrlKey_altKey_shiftKey_metaKey_altGraphKey_,  # noqa: B950
            9,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_keyLocation_ctrlKey_altKey_shiftKey_metaKey_altGraphKey_,  # noqa: B950
            10,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_keyLocation_ctrlKey_altKey_shiftKey_metaKey_,  # noqa: B950
            1,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_keyLocation_ctrlKey_altKey_shiftKey_metaKey_,  # noqa: B950
            2,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_keyLocation_ctrlKey_altKey_shiftKey_metaKey_,  # noqa: B950
            6,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_keyLocation_ctrlKey_altKey_shiftKey_metaKey_,  # noqa: B950
            7,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_keyLocation_ctrlKey_altKey_shiftKey_metaKey_,  # noqa: B950
            8,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_keyLocation_ctrlKey_altKey_shiftKey_metaKey_,  # noqa: B950
            9,
        )

    @min_os_level("10.10")
    def test_methods10_10(self):
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_location_ctrlKey_altKey_shiftKey_metaKey_altGraphKey_,  # noqa: B950
            1,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_location_ctrlKey_altKey_shiftKey_metaKey_altGraphKey_,  # noqa: B950
            2,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_location_ctrlKey_altKey_shiftKey_metaKey_altGraphKey_,  # noqa: B950
            6,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_location_ctrlKey_altKey_shiftKey_metaKey_altGraphKey_,  # noqa: B950
            7,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_location_ctrlKey_altKey_shiftKey_metaKey_altGraphKey_,  # noqa: B950
            8,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_location_ctrlKey_altKey_shiftKey_metaKey_altGraphKey_,  # noqa: B950
            9,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_location_ctrlKey_altKey_shiftKey_metaKey_altGraphKey_,  # noqa: B950
            10,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_location_ctrlKey_altKey_shiftKey_metaKey_,  # noqa: B950
            1,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_location_ctrlKey_altKey_shiftKey_metaKey_,  # noqa: B950
            2,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_location_ctrlKey_altKey_shiftKey_metaKey_,  # noqa: B950
            6,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_location_ctrlKey_altKey_shiftKey_metaKey_,  # noqa: B950
            7,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_location_ctrlKey_altKey_shiftKey_metaKey_,  # noqa: B950
            8,
        )
        self.assertArgIsBOOL(
            WebKit.DOMKeyboardEvent.initKeyboardEvent_canBubble_cancelable_view_keyIdentifier_location_ctrlKey_altKey_shiftKey_metaKey_,  # noqa: B950
            9,
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestWKContentRuleListStore(TestCase):
    @min_os_level("10.13")
    def testMethods(self):
        self.assertArgIsBlock(
            WebKit.WKContentRuleListStore.compileContentRuleListForIdentifier_encodedContentRuleList_completionHandler_,  # noqa: B950
            2,
            b"v@@",
        )
        self.assertArgIsBlock(
            WebKit.WKContentRuleListStore.lookUpContentRuleListForIdentifier_completionHandler_,
            1,
            b"v@@",
        )
        self.assertArgIsBlock(
            WebKit.WKContentRuleListStore.removeContentRuleListForIdentifier_completionHandler_,
            1,
            b"v@",
        )
        self.assertArgIsBlock(
            WebKit.WKContentRuleListStore.getAvailableContentRuleListIdentifiers_,
            0,
            b"v@",
        )



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMHTMLMenuElement(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMHTMLMenuElement.compact)
        self.assertArgIsBOOL(WebKit.DOMHTMLMenuElement.setCompact_, 0)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMXPathResult(TestCase):
    def testConstants(self):
        self.assertEqual(WebKit.DOM_ANY_TYPE, 0)
        self.assertEqual(WebKit.DOM_NUMBER_TYPE, 1)
        self.assertEqual(WebKit.DOM_STRING_TYPE, 2)
        self.assertEqual(WebKit.DOM_BOOLEAN_TYPE, 3)
        self.assertEqual(WebKit.DOM_UNORDERED_NODE_ITERATOR_TYPE, 4)
        self.assertEqual(WebKit.DOM_ORDERED_NODE_ITERATOR_TYPE, 5)
        self.assertEqual(WebKit.DOM_UNORDERED_NODE_SNAPSHOT_TYPE, 6)
        self.assertEqual(WebKit.DOM_ORDERED_NODE_SNAPSHOT_TYPE, 7)
        self.assertEqual(WebKit.DOM_ANY_UNORDERED_NODE_TYPE, 8)
        self.assertEqual(WebKit.DOM_FIRST_ORDERED_NODE_TYPE, 9)

    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMXPathResult.booleanValue)
        self.assertResultIsBOOL(WebKit.DOMXPathResult.invalidIteratorState)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMHTMLAreaElement(TestCase):
    def testMehods(self):
        self.assertResultIsBOOL(WebKit.DOMHTMLAreaElement.noHref)
        self.assertArgIsBOOL(WebKit.DOMHTMLAreaElement.setNoHref_, 0)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestWebBackForwardList(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.WebBackForwardList.containsItem_)



"""
A basic command-line tool that uses JavaScriptCore

See also <http://parmanoir.com/Taming_JavascriptCore_within_and_without_WebView>

TODO: This needs to be an example that does something useful
"""
import JavaScriptCore

with JavaScriptCore.autoreleasing(JavaScriptCore.JSGlobalContextCreate(None)) as ctx:

    script = JavaScriptCore.JSStringCreateWithUTF8CString(b"return new Array")
    fn, exc = JavaScriptCore.JSObjectMakeFunction(
        ctx, None, 0, None, script, None, 1, None
    )
    assert exc is None
    result, error = JavaScriptCore.JSObjectCallAsFunction(ctx, fn, None, 0, None, None)
    JavaScriptCore.JSStringRelease(script)

    # Result is now a reference to a JavaScript array.
    print(result)



from PyObjCTools.TestSupport import TestCase, min_sdk_level
import WebKit  # noqa: F401
import objc


class TestWebFrameLoadDelegateHelper(WebKit.NSObject):
    def webView_willPerformClientRedirectToURL_delay_fireDate_forFrame_(
        self, a, b, c, d, e
    ):
        pass


class TestWebFrameLoadDelegate(TestCase):
    @min_sdk_level("10.11")
    def testProtocols(self):
        objc.protocolNamed("WebFrameLoadDelegate")

    def testMethods(self):
        self.assertArgHasType(
            TestWebFrameLoadDelegateHelper.webView_willPerformClientRedirectToURL_delay_fireDate_forFrame_,  # noqa: B950
            2,
            objc._C_DBL,
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestDOMEvent(TestCase):
    def testConstants(self):
        self.assertEqual(WebKit.DOM_NONE, 0)
        self.assertEqual(WebKit.DOM_CAPTURING_PHASE, 1)
        self.assertEqual(WebKit.DOM_AT_TARGET, 2)
        self.assertEqual(WebKit.DOM_BUBBLING_PHASE, 3)

    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMEvent.bubbles)
        self.assertResultIsBOOL(WebKit.DOMEvent.cancelable)
        self.assertArgIsBOOL(WebKit.DOMEvent.initEvent_canBubbleArg_cancelableArg_, 1)
        self.assertArgIsBOOL(WebKit.DOMEvent.initEvent_canBubbleArg_cancelableArg_, 2)
        self.assertArgIsBOOL(WebKit.DOMEvent.initEvent___, 1)
        self.assertArgIsBOOL(WebKit.DOMEvent.initEvent___, 2)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(WebKit.DOMEvent.returnValue)
        self.assertArgIsBOOL(WebKit.DOMEvent.setReturnValue_, 0)
        self.assertResultIsBOOL(WebKit.DOMEvent.cancelBubble)
        self.assertArgIsBOOL(WebKit.DOMEvent.setCancelBubble_, 0)



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit
import objc


class TestWKNavigationDelegateHelper(WebKit.NSObject):
    def webView_decidePolicyForNavigationAction_decisionHandler_(self, w, a, h):
        pass

    def webView_decidePolicyForNavigationResponse_decisionHandler_(self, w, r, h):
        pass

    def webView_didReceiveAuthenticationChallenge_completionHandler_(self, w, c, h):
        pass

    def webView_authenticationChallenge_shouldAllowDeprecatedTLS_(self, w, c, h):
        pass


class TestWKNavigationDelegate(TestCase):
    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(WebKit.WKErrorDomain, str)

        self.assertEqual(WebKit.WKNavigationActionPolicyCancel, 0)
        self.assertEqual(WebKit.WKNavigationActionPolicyAllow, 1)

        self.assertEqual(WebKit.WKNavigationResponsePolicyCancel, 0)
        self.assertEqual(WebKit.WKNavigationResponsePolicyAllow, 1)

    @min_os_level("10.10")
    def testProtocols(self):
        p = objc.protocolNamed("WKNavigationDelegate")
        self.assertIsInstance(p, objc.formal_protocol)

        self.assertArgIsBlock(
            TestWKNavigationDelegateHelper.webView_decidePolicyForNavigationAction_decisionHandler_,  # noqa: B950
            2,
            objc._C_VOID + objc._C_NSInteger,
        )
        self.assertArgIsBlock(
            TestWKNavigationDelegateHelper.webView_decidePolicyForNavigationResponse_decisionHandler_,  # noqa: B950
            2,
            objc._C_VOID + objc._C_NSInteger,
        )
        self.assertArgIsBlock(
            TestWKNavigationDelegateHelper.webView_didReceiveAuthenticationChallenge_completionHandler_,  # noqa: B950
            2,
            objc._C_VOID + objc._C_NSInteger + objc._C_ID,
        )

    @min_os_level("10.16")
    def testMethods10_16(self):
        self.assertArgIsBlock(
            TestWKNavigationDelegateHelper.webView_authenticationChallenge_shouldAllowDeprecatedTLS_,
            2,
            b"vZ",
        )



"""
Some simple tests to check that the framework is properly wrapped.
"""
import objc
import WebKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level


class TestWebKit(TestCase):
    def testClasses(self):
        self.assertHasAttr(WebKit, "WebResource")
        self.assertIsInstance(WebKit.WebResource, objc.objc_class)

        self.assertHasAttr(WebKit, "DOMHTMLObjectElement")
        self.assertIsInstance(WebKit.DOMHTMLObjectElement, objc.objc_class)

    def testValues(self):
        self.assertHasAttr(WebKit, "DOM_CSS_PERCENTAGE")
        self.assertIsInstance(WebKit.DOM_CSS_PERCENTAGE, int)
        self.assertEqual(WebKit.DOM_CSS_PERCENTAGE, 2)

        self.assertHasAttr(WebKit, "DOM_CSS_VALUE_LIST")
        self.assertIsInstance(WebKit.DOM_CSS_VALUE_LIST, int)
        self.assertEqual(WebKit.DOM_CSS_VALUE_LIST, 2)

        self.assertHasAttr(WebKit, "WebViewInsertActionDropped")
        self.assertIsInstance(WebKit.WebViewInsertActionDropped, int)

    def testVariables(self):
        self.assertHasAttr(WebKit, "DOMRangeException")
        self.assertIsInstance(WebKit.DOMRangeException, str)

    @min_sdk_level("10.6")
    def testProtocols(self):
        objc.protocolNamed("DOMEventListener")
        objc.protocolNamed("DOMEventTarget")
        objc.protocolNamed("DOMNodeFilter")
        objc.protocolNamed("DOMXPathNSResolver")
        objc.protocolNamed("WebDocumentRepresentation")
        objc.protocolNamed("WebDocumentSearching")
        objc.protocolNamed("WebDocumentText")
        objc.protocolNamed("WebDocumentView")
        objc.protocolNamed("WebOpenPanelResultListener")
        objc.protocolNamed("WebPlugInViewFactory")
        objc.protocolNamed("WebPolicyDecisionListener")



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestDOMElement(TestCase):
    def testConstants(self):
        self.assertEqual(WebKit.DOM_ALLOW_KEYBOARD_INPUT, 1)

    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMElement.hasAttribute_)
        self.assertResultIsBOOL(WebKit.DOMElement.hasAttributeNS_localName_)
        self.assertArgIsBOOL(WebKit.DOMElement.scrollIntoView_, 0)
        self.assertArgIsBOOL(WebKit.DOMElement.scrollIntoViewIfNeeded_, 0)
        self.assertResultIsBOOL(WebKit.DOMElement.hasAttributeNS__)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(WebKit.DOMElement.contains_)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMCSSValue(TestCase):
    def testConstants(self):
        self.assertEqual(WebKit.DOM_CSS_INHERIT, 0)
        self.assertEqual(WebKit.DOM_CSS_PRIMITIVE_VALUE, 1)
        self.assertEqual(WebKit.DOM_CSS_VALUE_LIST, 2)
        self.assertEqual(WebKit.DOM_CSS_CUSTOM, 3)



"""
Script for building the example.

Usage:
    python3 setup.py py2app
"""
from setuptools import setup

plist = {
    "CFBundleDocumentTypes": [
        {
            "CFBundleTypeExtensions": ["mht"],
            "CFBundleTypeName": "Internet Explorer Web Archive",
            "CFBundleTypeRole": "Editor",
            "NSDocumentClass": "MHTDocument",
        }
    ]
}

setup(
    name="MHTViewer",
    app=["main.py"],
    data_files=["MainMenu.nib", "MHTDocument.nib"],
    options={"py2app": {"plist": plist}},
    setup_requires=["py2app", "pyobjc-framework-Cocoa", "pyobjc-framework-WebKit"],
)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestWebHistory(TestCase):
    def testConstants(self):
        self.assertIsInstance(WebKit.WebHistoryItemsAddedNotification, str)
        self.assertIsInstance(WebKit.WebHistoryItemsRemovedNotification, str)
        self.assertIsInstance(WebKit.WebHistoryAllItemsRemovedNotification, str)
        self.assertIsInstance(WebKit.WebHistoryLoadedNotification, str)
        self.assertIsInstance(WebKit.WebHistorySavedNotification, str)
        self.assertIsInstance(WebKit.WebHistoryItemsKey, str)

    def testMethods(self):
        self.assertResultIsBOOL(WebKit.WebHistory.loadFromURL_error_)
        self.assertArgIsOut(WebKit.WebHistory.loadFromURL_error_, 1)
        self.assertResultIsBOOL(WebKit.WebHistory.saveToURL_error_)
        self.assertArgIsOut(WebKit.WebHistory.saveToURL_error_, 1)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMHTMLOListElement(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMHTMLOListElement.compact)
        self.assertArgIsBOOL(WebKit.DOMHTMLOListElement.setCompact_, 0)



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestWKFindResult(TestCase):
    @min_os_level("10.16")
    def testMethods10_16(self):
        self.assertResultIsBOOL(WebKit.WKFindResult.matchFound)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMOverflowEvent(TestCase):
    def testConstants(self):
        self.assertEqual(WebKit.DOM_HORIZONTAL, 0)
        self.assertEqual(WebKit.DOM_VERTICAL, 1)
        self.assertEqual(WebKit.DOM_BOTH, 2)

    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMOverflowEvent.horizontalOverflow)
        self.assertResultIsBOOL(WebKit.DOMOverflowEvent.verticalOverflow)

        self.assertArgIsBOOL(
            WebKit.DOMOverflowEvent.initOverflowEvent_horizontalOverflow_verticalOverflow_,
            1,
        )
        self.assertArgIsBOOL(
            WebKit.DOMOverflowEvent.initOverflowEvent_horizontalOverflow_verticalOverflow_,
            2,
        )



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMEventException(TestCase):
    def testConstants(self):
        self.assertIsInstance(WebKit.DOMEventException, str)
        self.assertEqual(WebKit.DOM_UNSPECIFIED_EVENT_TYPE_ERR, 0)



import WebKit
from PyObjCTools.TestSupport import TestCase, min_os_level

NSAttributedStringCompletionHandler = b"v@@@"


class TestWKContentRuleListStore(TestCase):
    @min_os_level("10.15")
    def test_constants(self):
        self.assertIsInstance(WebKit.NSReadAccessURLDocumentOption, str)

    @min_os_level("10.15")
    def test_methods(self):
        self.assertArgIsBlock(
            WebKit.NSAttributedString.loadFromHTMLWithRequest_options_completionHandler_,
            2,
            NSAttributedStringCompletionHandler,
        )
        self.assertArgIsBlock(
            WebKit.NSAttributedString.loadFromHTMLWithFileURL_options_completionHandler_,
            2,
            NSAttributedStringCompletionHandler,
        )
        self.assertArgIsBlock(
            WebKit.NSAttributedString.loadFromHTMLWithString_options_completionHandler_,
            2,
            NSAttributedStringCompletionHandler,
        )
        self.assertArgIsBlock(
            WebKit.NSAttributedString.loadFromHTMLWithData_options_completionHandler_,
            2,
            NSAttributedStringCompletionHandler,
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestWKWebsiteDataRecord(TestCase):
    @min_os_level("10.11")
    def testConstants10_10(self):
        self.assertIsInstance(WebKit.WKWebsiteDataTypeDiskCache, str)
        self.assertIsInstance(WebKit.WKWebsiteDataTypeMemoryCache, str)
        self.assertIsInstance(WebKit.WKWebsiteDataTypeOfflineWebApplicationCache, str)
        self.assertIsInstance(WebKit.WKWebsiteDataTypeCookies, str)
        self.assertIsInstance(WebKit.WKWebsiteDataTypeSessionStorage, str)
        self.assertIsInstance(WebKit.WKWebsiteDataTypeLocalStorage, str)
        self.assertIsInstance(WebKit.WKWebsiteDataTypeWebSQLDatabases, str)
        self.assertIsInstance(WebKit.WKWebsiteDataTypeIndexedDBDatabases, str)

    @min_os_level("10.13.4")
    def testConstants10_13_4(self):
        self.assertIsInstance(WebKit.WKWebsiteDataTypeFetchCache, str)
        self.assertIsInstance(WebKit.WKWebsiteDataTypeServiceWorkerRegistrations, str)



import JavaScriptCore
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestJSTypedArray(TestCase):
    @min_os_level("10.12")
    def test_functions(self):
        self.assertResultHasType(
            JavaScriptCore.JSObjectMakeTypedArray,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArray,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(JavaScriptCore.JSObjectMakeTypedArray, 1, objc._C_UINT)
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArray, 2, objc._C_ULNG
        )  # size_t
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArray,
            3,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithBytesNoCopy,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithBytesNoCopy,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithBytesNoCopy, 1, objc._C_UINT
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithBytesNoCopy, 2, b"n^v"
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithBytesNoCopy, 3, objc._C_ULNG
        )  # size_t
        self.assertArgIsFunction(
            JavaScriptCore.JSObjectMakeTypedArrayWithBytesNoCopy, 4, b"v^v^v", True
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithBytesNoCopy, 5, b"^v"
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithBytesNoCopy,
            6,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithArrayBuffer,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithArrayBuffer,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithArrayBuffer, 1, objc._C_UINT
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithArrayBuffer,
            2,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithArrayBuffer,
            3,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithArrayBufferAndOffset,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithArrayBufferAndOffset,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithArrayBufferAndOffset,
            1,
            objc._C_UINT,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithArrayBufferAndOffset,
            2,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithArrayBufferAndOffset,
            3,
            objc._C_ULNG,
        )  # size_t
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithArrayBufferAndOffset,
            4,
            objc._C_ULNG,
        )  # size_t
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeTypedArrayWithArrayBufferAndOffset,
            5,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(JavaScriptCore.JSObjectGetTypedArrayBytesPtr, b"^v")
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetTypedArrayBytesPtr,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetTypedArrayBytesPtr,
            1,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetTypedArrayBytesPtr,
            2,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectGetTypedArrayLength, objc._C_ULNG
        )  # size_t
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetTypedArrayLength,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetTypedArrayLength,
            1,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetTypedArrayLength,
            2,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectGetTypedArrayByteLength, objc._C_ULNG
        )  # size_t
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetTypedArrayByteLength,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetTypedArrayByteLength,
            1,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetTypedArrayByteLength,
            2,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectGetTypedArrayByteOffset, objc._C_ULNG
        )  # size_t
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetTypedArrayByteOffset,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetTypedArrayByteOffset,
            1,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetTypedArrayByteOffset,
            2,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectGetTypedArrayBuffer,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetTypedArrayBuffer,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetTypedArrayBuffer,
            1,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetTypedArrayBuffer,
            2,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(
            JavaScriptCore.JSObjectMakeArrayBufferWithBytesNoCopy,
            JavaScriptCore.JSObjectRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeArrayBufferWithBytesNoCopy,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeArrayBufferWithBytesNoCopy, 1, b"n^v"
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeArrayBufferWithBytesNoCopy, 2, objc._C_ULNG
        )  # size_t
        self.assertArgIsFunction(
            JavaScriptCore.JSObjectMakeArrayBufferWithBytesNoCopy, 3, b"v^v^v", True
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeArrayBufferWithBytesNoCopy, 4, b"^v"
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectMakeArrayBufferWithBytesNoCopy,
            5,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(JavaScriptCore.JSObjectGetArrayBufferBytesPtr, b"^v")
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetArrayBufferBytesPtr,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetArrayBufferBytesPtr,
            1,
            JavaScriptCore.JSValueRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetArrayBufferBytesPtr,
            2,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )

        self.assertResultHasType(JavaScriptCore.JSObjectGetArrayBufferByteLength, b"L")
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetArrayBufferByteLength,
            0,
            JavaScriptCore.JSContextRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetArrayBufferByteLength,
            1,
            JavaScriptCore.JSValueRef.__typestr__,
        )
        self.assertArgHasType(
            JavaScriptCore.JSObjectGetArrayBufferByteLength,
            2,
            b"o^" + JavaScriptCore.JSValueRef.__typestr__,
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestWKNavigationAction(TestCase):
    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(WebKit.WKNavigationTypeLinkActivated, 0)
        self.assertEqual(WebKit.WKNavigationTypeFormSubmitted, 1)
        self.assertEqual(WebKit.WKNavigationTypeBackForward, 2)
        self.assertEqual(WebKit.WKNavigationTypeReload, 3)
        self.assertEqual(WebKit.WKNavigationTypeFormResubmitted, 4)
        self.assertEqual(WebKit.WKNavigationTypeOther, -1)



import pydoc

__all__ = ["gethtmldoc"]


def gethtmldoc(thing, forceload=0):
    obj, name = pydoc.resolve(thing, forceload)
    page = pydoc.html.page(pydoc.describe(obj), pydoc.html.document(obj, name))
    return page



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestWKError(TestCase):
    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(WebKit.WKErrorDomain, str)

        self.assertEqual(WebKit.WKErrorUnknown, 1)
        self.assertEqual(WebKit.WKErrorWebContentProcessTerminated, 2)
        self.assertEqual(WebKit.WKErrorWebViewInvalidated, 3)
        self.assertEqual(WebKit.WKErrorJavaScriptExceptionOccurred, 4)
        self.assertEqual(WebKit.WKErrorJavaScriptResultTypeIsUnsupported, 5)
        self.assertEqual(WebKit.WKErrorContentRuleListStoreCompileFailed, 6)
        self.assertEqual(WebKit.WKErrorContentRuleListStoreLookUpFailed, 7)
        self.assertEqual(WebKit.WKErrorContentRuleListStoreRemoveFailed, 8)
        self.assertEqual(WebKit.WKErrorContentRuleListStoreVersionMismatch, 9)
        self.assertEqual(WebKit.WKErrorAttributedStringContentFailedToLoad, 10)
        self.assertEqual(WebKit.WKErrorAttributedStringContentLoadTimedOut, 11)
        self.assertEqual(WebKit.WKErrorJavaScriptInvalidFrameTarget, 12)
        self.assertEqual(WebKit.WKErrorNavigationAppBoundDomain, 13)
        self.assertEqual(WebKit.WKErrorJavaScriptAppBoundDomain, 14)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMTreeWalker(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMTreeWalker.expandEntityReferences)



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestWebPagePreferences(TestCase):
    def test_constants(self):
        self.assertEqual(WebKit.WKContentModeRecommended, 0)
        self.assertEqual(WebKit.WKContentModeMobile, 1)
        self.assertEqual(WebKit.WKContentModeDesktop, 2)

    @min_os_level("10.16")
    def test_methods10_16(self):
        self.assertResultIsBOOL(WebKit.WKWebpagePreferences.allowsContentJavaScript)
        self.assertArgIsBOOL(WebKit.WKWebpagePreferences.setAllowsContentJavaScript_, 0)



import email

from Cocoa import NSURL, NSData, NSString
from WebKit import WebArchive, WebResource


# def loadMHT(filename):
#     """
#     Load a .HMT HTML archive and return the WebArchive representation.
#     """
#     return HMTLoad(filename).asWebArchive()


class MHTLoader(object):
    """
    A loader for .mht files, and archive format used by MS Internet Explorer
    on Windows.
    """

    def __init__(self, filename):
        self.filename = filename

        # root of the archive (index into self.parts)
        self.root = None

        # filename -> (content-type, data)
        self.parts = {}

        self.loadFile(filename)

    def loadFile(self, filename):
        with open(filename, "r") as fp:
            msg = email.message_from_file(fp)

        for part in msg.walk():
            if part.get_content_maintype() == "multipart":
                continue

            filename = part.get("Content-Location")
            contentType = part.get_content_type()
            data = part.get_payload(decode=True)

            self.parts[filename] = (contentType, data)
            if self.root is None:
                self.root = filename

    def fixupURL(self, url):
        # IE creates MHT files with file: URLS containing backslashes,
        # NSURL insists that those are invalid, replace backslashes by
        # forward slashes.
        if url.startswith("file:"):
            return url.replace("\\", "/")
        else:
            return url

    def asWebArchive(self):
        """
        Convert the MHT archive to a webarchive.
        """
        rootType, rootText = self.parts[self.root]
        pageResource = WebResource.alloc().initWithData_URL_MIMEType_textEncodingName_frameName_(  # noqa: B950
            NSData.dataWithBytes_length_(rootText.replace(b"\\", b"/"), len(rootText)),
            NSURL.URLWithString_(self.fixupURL(self.root)),
            NSString.stringWithString_(rootType),
            None,
            None,
        )

        resources = []
        for url in self.parts:
            if url == self.root:
                continue

            tp, data = self.parts[url]
            resources.append(
                WebResource.alloc().initWithData_URL_MIMEType_textEncodingName_frameName_(
                    NSData.dataWithBytes_length_(data, len(data)),
                    NSURL.URLWithString_(self.fixupURL(url)),
                    NSString.stringWithString_(tp),
                    None,
                    None,
                )
            )

        return WebArchive.alloc().initWithMainResource_subresources_subframeArchives_(
            pageResource, resources, None
        )


def main():
    # Testing...
    p = MHTLoader("python-home.mht")
    a = p.asWebArchive()
    with open("python-home.webarchive", "wb") as fp:
        fp.write(a.data().bytes())


if __name__ == "__main__":
    main()



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestWebKitErrors(TestCase):
    def testConstants(self):
        self.assertIsInstance(WebKit.WebKitErrorDomain, str)
        self.assertIsInstance(WebKit.WebKitErrorMIMETypeKey, str)
        self.assertIsInstance(WebKit.WebKitErrorPlugInNameKey, str)
        self.assertIsInstance(WebKit.WebKitErrorPlugInPageURLStringKey, str)

        self.assertEqual(WebKit.WebKitErrorCannotShowMIMEType, 100)
        self.assertEqual(WebKit.WebKitErrorCannotShowURL, 101)
        self.assertEqual(WebKit.WebKitErrorFrameLoadInterruptedByPolicyChange, 102)
        self.assertEqual(WebKit.WebKitErrorCannotFindPlugIn, 200)
        self.assertEqual(WebKit.WebKitErrorCannotLoadPlugIn, 201)
        self.assertEqual(WebKit.WebKitErrorJavaUnavailable, 202)
        self.assertEqual(WebKit.WebKitErrorBlockedPlugInVersion, 203)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMHTMLScriptElement(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMHTMLScriptElement.defer)
        self.assertArgIsBOOL(WebKit.DOMHTMLScriptElement.setDefer_, 0)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMTraversal(TestCase):
    def testMethods(self):
        self.assertArgIsBOOL(
            WebKit.DOMDocument.createNodeIterator_whatToShow_filter_expandEntityReferences_,
            3,
        )
        self.assertArgIsBOOL(
            WebKit.DOMDocument.createTreeWalker_whatToShow_filter_expandEntityReferences_,
            3,
        )
        self.assertArgIsBOOL(WebKit.DOMDocument.createNodeIterator____, 3)
        self.assertArgIsBOOL(WebKit.DOMDocument.createTreeWalker____, 3)



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMUIEvent(TestCase):
    def testMethods(self):
        self.assertArgIsBOOL(
            WebKit.DOMUIEvent.initUIEvent_canBubble_cancelable_view_detail_, 1
        )
        self.assertArgIsBOOL(
            WebKit.DOMUIEvent.initUIEvent_canBubble_cancelable_view_detail_, 2
        )
        self.assertArgIsBOOL(WebKit.DOMUIEvent.initUIEvent_____, 1)
        self.assertArgIsBOOL(WebKit.DOMUIEvent.initUIEvent_____, 2)



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestDOMProgressEvent(TestCase):
    @min_os_level("10.6")
    def testMethod10_6(self):
        self.assertResultIsBOOL(WebKit.DOMProgressEvent.lengthComputable)



import sys

from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestWKWebViewConfiguration(TestCase):
    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(
            WebKit.WKWebViewConfiguration.suppressesIncrementalRendering
        )
        self.assertArgIsBOOL(
            WebKit.WKWebViewConfiguration.setSuppressesIncrementalRendering_, 0
        )

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(
            WebKit.WKWebViewConfiguration.allowsAirPlayForMediaPlayback
        )
        self.assertArgIsBOOL(
            WebKit.WKWebViewConfiguration.setAllowsAirPlayForMediaPlayback_, 0
        )

    @min_os_level("10.16")
    def testMethods10_16(self):
        self.assertResultIsBOOL(
            WebKit.WKWebViewConfiguration.limitsNavigationsToAppBoundDomains
        )

    def testConstants(self):
        self.assertEqual(WebKit.WKUserInterfaceDirectionPolicyContent, 0)
        self.assertEqual(WebKit.WKUserInterfaceDirectionPolicySystem, 1)

        self.assertEqual(WebKit.WKAudiovisualMediaTypeNone, 0)
        self.assertEqual(WebKit.WKAudiovisualMediaTypeAudio, 1 << 0)
        self.assertEqual(WebKit.WKAudiovisualMediaTypeVideo, 1 << 1)

        # The entire enum is only available in 64-bit code.
        self.assertEqual(WebKit.WKAudiovisualMediaTypeAll, sys.maxsize * 2 + 1)



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit


class TestWKPreferences(TestCase):
    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(WebKit.WKPreferences.javaScriptEnabled)
        self.assertArgIsBOOL(WebKit.WKPreferences.setJavaScriptEnabled_, 0)
        self.assertResultIsBOOL(
            WebKit.WKPreferences.javaScriptCanOpenWindowsAutomatically
        )
        self.assertArgIsBOOL(
            WebKit.WKPreferences.setJavaScriptCanOpenWindowsAutomatically_, 0
        )
        self.assertResultIsBOOL(WebKit.WKPreferences.javaEnabled)
        self.assertArgIsBOOL(WebKit.WKPreferences.setJavaEnabled_, 0)
        self.assertResultIsBOOL(WebKit.WKPreferences.plugInsEnabled)
        self.assertArgIsBOOL(WebKit.WKPreferences.setPlugInsEnabled_, 0)

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(WebKit.WKPreferences.tabFocusesLinks)
        self.assertArgIsBOOL(WebKit.WKPreferences.setTabFocusesLinks_, 0)

    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertResultIsBOOL(WebKit.WKPreferences.isFraudulentWebsiteWarningEnabled)
        self.assertArgIsBOOL(
            WebKit.WKPreferences.setFraudulentWebsiteWarningEnabled_, 0
        )



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMHTMLOptionElement(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMHTMLOptionElement.defaultSelected)
        self.assertArgIsBOOL(WebKit.DOMHTMLOptionElement.setDefaultSelected_, 0)
        self.assertResultIsBOOL(WebKit.DOMHTMLOptionElement.disabled)
        self.assertArgIsBOOL(WebKit.DOMHTMLOptionElement.setDisabled_, 0)
        self.assertResultIsBOOL(WebKit.DOMHTMLOptionElement.selected)
        self.assertArgIsBOOL(WebKit.DOMHTMLOptionElement.setSelected_, 0)



from PyObjCTools.TestSupport import TestCase, min_os_level
import WebKit  # noqa: F401
import objc


class TestWKScriptMessageHandler(TestCase):
    @min_os_level("10.10")
    def testProtocols10_10(self):
        p = objc.protocolNamed("WKScriptMessageHandler")
        self.assertIsInstance(p, objc.formal_protocol)



#!/usr/bin/env python
"""
This script will rewrite the macho headers of the frameworks in the
nightly webkit snapshot, to make sure PyObjC can use them.
"""
import os
import shutil

from macholib.MachO import MachO


def rewriteFramework(framework, frameworkMap):

    basename = os.path.splitext(os.path.basename(framework))[0]
    dyld = os.path.abspath(os.path.join(framework, basename))

    macho = MachO(dyld)

    def changefunc(key):
        if key == dyld:
            return dyld

        dirname, filename = os.path.split(key)
        return frameworkMap.get(filename)

    macho.rewriteLoadCommands(changefunc)
    macho.write(open(dyld, "rb+"))


def rewriteFrameworksInDirectory(dirname):
    frameworks = [fn for fn in os.listdir(dirname) if fn.endswith(".framework")]
    mapping = {}
    for fn in frameworks:
        mapping[os.path.splitext(fn)[0]] = os.path.join(
            os.path.abspath(dirname), fn, os.path.splitext(fn)[0]
        )

    for fn in frameworks:
        rewriteFramework(fn, mapping)


def extractWebKitApp(pathToApp, outputDir):
    resources = os.path.join(pathToApp, "Contents", "Resources")
    frameworks = [fn for fn in os.listdir(resources) if fn.endswith(".framework")]
    for framework in frameworks:
        if os.path.exists(os.path.join(outputDir, framework)):
            shutil.rmtree(os.path.join(outputDir, framework))

        shutil.copytree(
            os.path.join(resources, framework),
            os.path.join(outputDir, framework),
            symlinks=True,
        )

        rewriteFrameworksInDirectory(outputDir)


def main():
    extractWebKitApp("WebKit.app", ".")


if __name__ == "__main__":
    main()



from PyObjCTools.TestSupport import TestCase
import WebKit


class TestDOMHTMLFrameElement(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(WebKit.DOMHTMLFrameElement.noResize)
        self.assertArgIsBOOL(WebKit.DOMHTMLFrameElement.setNoResize_, 0)



import JavaScriptCore
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestJSContext(TestCase):
    @min_os_level("10.9")
    def test_classes(self):
        self.assertHasAttr(JavaScriptCore, "JSContext")

        self.assertResultIsBlock(JavaScriptCore.JSContext.exceptionHandler, b"v@@")
        self.assertArgIsBlock(JavaScriptCore.JSContext.setExceptionHandler_, 0, b"v@@")



from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc
import WebKit  # noqa: F401


class TestWebResourceLoadDelegateHelper(WebKit.NSObject):
    def webView_resource_didReceiveContentLength_fromDataSource_(self, a, b, c, d):
        pass


class TestWebResourceLoadDelegate(TestCase):
    @min_sdk_level("10.11")
    def testProtocols(self):
        objc.protocolNamed("WebResourceLoadDelegate")

    def testMethods(self):
        self.assertArgHasType(
            TestWebResourceLoadDelegateHelper.webView_resource_didReceiveContentLength_fromDataSource_,  # noqa: B950
            2,
            objc._C_NSInteger,
        )



"""
Wrappers for the "WebKit" and "JavaScriptCore" frameworks on macOS. The
WebKit framework contains the views and support classes for creating a
browser. The JavaScriptCore framework implements a JavaScript interpreter.

These wrappers don't include documentation, please check Apple's documention
for information on how to use these frameworks and PyObjC's documentation
for general tips and tricks regarding the translation between Python
and (Objective-)C frameworks
"""
import os

from pyobjc_setup import Extension, setup

VERSION = '7.0.1'

setup(
    name="pyobjc-framework-WebKit",
    description="Wrappers for the framework WebKit on macOS",
    packages=["WebKit", "JavaScriptCore"],
    ext_modules=[
        Extension(
            "WebKit._WebKit",
            ["Modules/_WebKit.m"],
            extra_link_args=["-framework", "WebKit"],
            py_limited_api=True,
            depends=[
                os.path.join("Modules", fn)
                for fn in os.listdir("Modules")
                if fn.startswith("_WebKit")
            ],
        )
    ],
    version=VERSION,
    install_requires=["pyobjc-core>=" + VERSION, "pyobjc-framework-Cocoa>=" + VERSION],
    long_description=__doc__,
    options={"bdist_wheel": {"py_limited_api": "cp36"}},
)
